{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": true
   },
   "source": [
    "import random\n",
    "import string\n",
    "import copy\n",
    "\n",
    "class Building():\n",
    "    def __init__(self, num_of_floor, num_of_passenger):\n",
    "        self.num_of_floor = num_of_floor\n",
    "        self.num_of_passenger = num_of_passenger\n",
    "        self.passengers = [] #list of all passengers\n",
    "        self.random_passengers()   #generate random passengers\n",
    "\n",
    "    def random_passengers(self):\n",
    "        for i in range(self.num_of_passenger):\n",
    "            p = Passenger(random.randrange(self.num_of_floor), random.randrange(self.num_of_floor)) #generates an origin and a destination\n",
    "\n",
    "            # make sure the origin and destination is different\n",
    "            while p.origin == p.destination:\n",
    "                p.destination = random.randrange(self.num_of_floor)\n",
    "\n",
    "            self.passengers.append(p) #add random passenger to building\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "class Passenger():\n",
    "    def __init__(self, origin, destination):\n",
    "        self.name = \"\".join(random.choice(string.ascii_uppercase)) +\\\n",
    "                    \"\".join(random.choice(string.ascii_lowercase) for _ in range(random.randrange(2, 8)))\n",
    "                    # a random name for the passenger for console display\n",
    "        self.origin = origin\n",
    "        self.destination = destination\n",
    "        self.deliver = 0   # marker of if the passenger is delivered, 0=undelivered 1=delivered\n",
    "        self.cost = 0   # the cost to deliver the passenger\n",
    "\n",
    "    def compute_cost(self, movement):\n",
    "        self.cost = movement - abs(self.origin - self.destination)\n",
    "        print(\"Drop off\", self.name, \"at floor\", self.destination, \"with cost\", self.cost)\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "class Elevator():\n",
    "    def __init__(self, capacity, position=0, direction=1):\n",
    "        self.capacity = capacity   # max allowed people in the elevator\n",
    "        self.position = position   # current position of the elevator, defalut to 0\n",
    "        self.movement = 0   # count the total movement of the elevator, used to calculate cost function\n",
    "        self.direction = direction   # last moving direction of the elevator, 0=down 1=up, default to 1\n",
    "        self.carrying = []   # a collection of passengers currently inside the elevator\n",
    "\n",
    "    def move(self):\n",
    "        # move one floor following the given direction\n",
    "        if self.direction == 1:\n",
    "            self.position += 1\n",
    "        else:\n",
    "            self.position -= 1\n",
    "\n",
    "        self.movement += 1   # increment the total movement by 1 as the elevator has moved one more step\n",
    "\n",
    "    def drop_passenger(self):\n",
    "        # go through all passengers on the elevator to check if they reached their destination\n",
    "        dropped = []\n",
    "        for key, object in enumerate(self.carrying):\n",
    "            if object.destination == self.position: #mark if passenger is at destination floor\n",
    "                dropped.append(key) #add passenger to be dropped\n",
    "                object.compute_cost(self.movement)\n",
    "        \n",
    "        if dropped: #removed dropped passengers from elevator\n",
    "            self.carrying = [p for ind, p in enumerate(self.carrying) if ind not in dropped]\n",
    "\n",
    "\n",
    "    def reverse(self):\n",
    "        if self.direction == 1:\n",
    "            self.direction = 0\n",
    "        else:\n",
    "            self.direction = 1\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "\n",
    "# Strategy 1 has 3 principles:\n",
    "# 1. The elevator will always favour its current moving direction.\n",
    "# 2. Unless reached its capacity, the elevator will take all encountered passengers that are going in the same direction.\n",
    "# 3. The elevator will reverse its current moving direction if:\n",
    "#    1. There is no passenger currently inside the elevator.\n",
    "#    2. There is no undelivered passenger in the elevator's current direction\n",
    "\n",
    "def strategy_1(b, e): #inputs a building and an elevator\n",
    "    awaiting = copy.copy(b.passengers) #list of passengers in the building \n",
    "    e.carrying = [] #reset elevator\n",
    "    while len(awaiting) > 0 or len(e.carrying) > 0: #while there are people to be picked up or people to deliver\n",
    "        print(\"The elevator is at\", e.position, \"carrying\", len(e.carrying), \"passengers\")\n",
    "\n",
    "        # at each floor, first check if there is a passenger to drop\n",
    "        e.drop_passenger()\n",
    "\n",
    "        # if the elevator still has capacity, check if there is passenger to carry\n",
    "        if len(awaiting) > 0 and len(e.carrying) < e.capacity:\n",
    "            DEL = []\n",
    "            for key, object in enumerate(awaiting): #loop through all waiting passengers in the building\n",
    "                if e.position == object.origin:\n",
    "                    if ((object.destination - object.origin) > 0) == e.direction:   # if the elevator is going in the direction the passenger wants to go\n",
    "                        e.carrying.append(object) #pick up passenger\n",
    "                        DEL.append(key) # add passenger to be removed from elevator calls\n",
    "                        print(\"Picking up\", object.name, \"at floor\", e.position)\n",
    "                        if len(e.carrying) == e.capacity:\n",
    "                            break   # when capacity is reached, stop checking if there are passengers to pick up (break for loop)\n",
    "            if DEL:\n",
    "                awaiting = [p for ind, p in enumerate(awaiting) if ind not in DEL] #remove dropped passengers from awaiting list\n",
    "\n",
    "        # decide if to reverse\n",
    "        if len(e.carrying) == 0: #if the elevator isn't carrying any passengers\n",
    "            reverse = 1 #reverse direction unless\n",
    "            for object in awaiting: #there are passengers to pick up in the current direction\n",
    "                if ((object.origin - e.position) > 0) == e.direction:\n",
    "                    reverse = 0\n",
    "                    break\n",
    "            if reverse == 1:\n",
    "                e.reverse()\n",
    "\n",
    "        # if there's a passenger waiting in the same direction, move one floor; if not, keep the position and re-evaluate\n",
    "        if len(e.carrying) > 0 or reverse == 0:\n",
    "            e.move()\n",
    "\n",
    "        # break in case of infinite loop\n",
    "        if e.movement > 10000: break\n",
    "\n",
    "    # calculate the total cost\n",
    "    total_cost = 0\n",
    "    for _, object in enumerate(b.passengers):\n",
    "        print(object.name, \"going from floor\", object.origin, \"to floor\", object.destination,\"have a cost of\", object.cost)\n",
    "        total_cost += object.cost\n",
    "    print(\"The total cost is\", total_cost)\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "\n",
    "#Strategy 2:\n",
    "# Elevator goes all the way up and then down and opens door at every floor\n",
    "def strategy_2(b, e): #inputs a building and an elevator\n",
    "    awaiting = copy.copy(b.passengers) #list of passengers in the building \n",
    "    e.carrying = [] #reset elevator\n",
    "    while len(awaiting) > 0 or len(e.carrying) > 0: #while there are people to be picked up or people to deliver\n",
    "        print(\"The elevator is at\", e.position, \"carrying\", len(e.carrying), \"passengers\")\n",
    "\n",
    "        # at each floor, first check if there is a passenger to drop\n",
    "        e.drop_passenger()\n",
    "        \n",
    "        # if the elevator still has capacity, check if there is passenger to carry\n",
    "        if len(awaiting) > 0 and len(e.carrying) < e.capacity:\n",
    "            DEL = []\n",
    "            for key, object in enumerate(awaiting): #loop through all waiting passengers in the building\n",
    "                print(e.position, object.name, object.origin)\n",
    "                if e.position == object.origin:\n",
    "                    e.carrying.append(object) #pick up passenger\n",
    "                    DEL.append(key) # add passenger to be removed from elevator calls\n",
    "                    print(\"Picking up\", object.name, \"at floor\", e.position)\n",
    "                    if len(e.carrying) == e.capacity:\n",
    "                        break   # when capacity is reached, stop checking if there are passengers to pick up (break for loop)\n",
    "            if DEL:\n",
    "                awaiting = [p for ind, p in enumerate(awaiting) if ind not in DEL] #remove dropped passengers from awaiting list\n",
    "\n",
    "        # decide if to reverse\n",
    "        if e.position == b.num_of_floor: #if at the top, go down\n",
    "            e.direction = 0\n",
    "        elif e.position == 0: #if on ground floor, go back up\n",
    "            e.direction = 1\n",
    "        \n",
    "        #move elevator in up or down direction\n",
    "        e.move()\n",
    "\n",
    "        # break in case of infinite loop\n",
    "        if e.movement > 100: break\n",
    "\n",
    "    # calculate the total cost\n",
    "    total_cost = 0\n",
    "    for _, object in enumerate(b.passengers):\n",
    "        print(object.name, \"going from floor\", object.origin, \"to floor\", object.destination,\"have a cost of\", object.cost)\n",
    "        total_cost += object.cost\n",
    "    print(\"The total cost is\", total_cost)\n",
    "\n",
    "###########################################################################"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "ename": "NameError",
     "evalue": "name 'Building' is not defined",
     "output_type": "error",
     "traceback": [
      "\u001b[0;31m---------------------------------------------------------------------------\u001b[0m",
      "\u001b[0;31mNameError\u001b[0m                                 Traceback (most recent call last)",
      "\u001b[0;32m<ipython-input-1-f2f634e0e7a0>\u001b[0m in \u001b[0;36m<module>\u001b[0;34m()\u001b[0m\n\u001b[0;32m----> 1\u001b[0;31m \u001b[0mbuilding_1\u001b[0m \u001b[0;34m=\u001b[0m \u001b[0mBuilding\u001b[0m\u001b[0;34m(\u001b[0m\u001b[0;36m50\u001b[0m\u001b[0;34m,\u001b[0m \u001b[0;36m25\u001b[0m\u001b[0;34m)\u001b[0m\u001b[0;34m\u001b[0m\u001b[0m\n\u001b[0m\u001b[1;32m      2\u001b[0m \u001b[0melevator_1\u001b[0m \u001b[0;34m=\u001b[0m \u001b[0mElevator\u001b[0m\u001b[0;34m(\u001b[0m\u001b[0;36m10\u001b[0m\u001b[0;34m)\u001b[0m\u001b[0;34m\u001b[0m\u001b[0m\n\u001b[1;32m      3\u001b[0m \u001b[0;34m\u001b[0m\u001b[0m\n\u001b[1;32m      4\u001b[0m \u001b[0mstrategy_1\u001b[0m\u001b[0;34m(\u001b[0m\u001b[0mcopy\u001b[0m\u001b[0;34m.\u001b[0m\u001b[0mcopy\u001b[0m\u001b[0;34m(\u001b[0m\u001b[0mbuilding_1\u001b[0m\u001b[0;34m)\u001b[0m\u001b[0;34m,\u001b[0m \u001b[0mcopy\u001b[0m\u001b[0;34m.\u001b[0m\u001b[0mcopy\u001b[0m\u001b[0;34m(\u001b[0m\u001b[0melevator_1\u001b[0m\u001b[0;34m)\u001b[0m\u001b[0;34m)\u001b[0m\u001b[0;34m\u001b[0m\u001b[0m\n",
      "\u001b[0;31mNameError\u001b[0m: name 'Building' is not defined"
     ]
    }
   ],
   "source": [
    "building_1 = Building(50, 25)\n",
    "elevator_1 = Elevator(10)\n",
    "\n",
    "strategy_1(copy.copy(building_1), copy.copy(elevator_1))\n",
    "strategy_2(copy.copy(building_1), copy.copy(elevator_1))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 65,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "The elevator is at 0 carrying 0 passengers\n",
      "The elevator is at 1 carrying 0 passengers\n",
      "Picking up Cqfdsmil at floor 1\n",
      "The elevator is at 2 carrying 1 passengers\n",
      "The elevator is at 3 carrying 1 passengers\n",
      "The elevator is at 4 carrying 1 passengers\n",
      "The elevator is at 5 carrying 1 passengers\n",
      "The elevator is at 6 carrying 1 passengers\n",
      "The elevator is at 7 carrying 1 passengers\n",
      "Picking up Hjxfqz at floor 7\n",
      "The elevator is at 8 carrying 2 passengers\n",
      "Drop off Hjxfqz at floor 8 with cost 7\n",
      "The elevator is at 9 carrying 1 passengers\n",
      "Picking up Ahnqujya at floor 9\n",
      "The elevator is at 10 carrying 2 passengers\n",
      "The elevator is at 9 carrying 2 passengers\n",
      "The elevator is at 8 carrying 2 passengers\n",
      "The elevator is at 7 carrying 2 passengers\n",
      "The elevator is at 6 carrying 2 passengers\n",
      "The elevator is at 5 carrying 2 passengers\n",
      "The elevator is at 4 carrying 2 passengers\n",
      "The elevator is at 3 carrying 2 passengers\n",
      "The elevator is at 2 carrying 2 passengers\n",
      "Drop off Ahnqujya at floor 2 with cost 11\n",
      "The elevator is at 1 carrying 1 passengers\n",
      "The elevator is at 0 carrying 1 passengers\n",
      "Drop off Cqfdsmil at floor 0 with cost 19\n",
      "Cqfdsmil going from floor 1 to floor 0 have a cost of 19\n",
      "Ahnqujya going from floor 9 to floor 2 have a cost of 11\n",
      "Hjxfqz going from floor 7 to floor 8 have a cost of 7\n",
      "The total cost is 37\n"
     ]
    }
   ],
   "source": [
    "strategy_2(copy.copy(building_1), copy.copy(elevator_1))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (Anaconda)",
   "language": "python",
   "name": "anaconda3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.5.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}